package net.northking.cctp.debuger.service;

import javax.websocket.Session;

public interface MobileScriptDebugService {

    void addDebugSession(String sessionId, Session session);

    void removeDebugSession(String sessionId, Session session);

    void processWebMsg(String msg);

    void removeEngineSession(String sessionId);

    void exitDeviceUsage(String sessionId);

    String removeUserToken(String sessionId);

    void addUserToken(String sessionId, String userToken);

    String getUserToken(String sessionId);

}
